#include <limits.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <curl/curl.h>
#include <stdio.h>
#include <wchar.h>
#include "fmt.h"
#include "tags.h"
#include "listento.h"
#include "config.h"

#define LISTENTO_URL "http://magic.listen-to.com"
#define LISTENTO_CLI_ID "ListenTo_xmms"

static int	lt_errors;

static char	lt_curl_errbuf[CURL_ERROR_SIZE],
		*lt_username,
		*lt_password;

char *adjust_spaces(char *tmp)
{
	char *i, *ret;
	int j;
	for(i = tmp; i < tmp + strlen(tmp); i++)
	{
		if(*i == ' ')
		{
			j = i - tmp;
			ret = malloc(strlen(tmp) + 1);
			memcpy(ret, tmp, strlen(tmp) + 1);
			free(tmp);
			tmp = malloc(strlen(ret) + 3);
			memcpy(tmp, ret, strlen(ret) + 1);
			free(ret);
			i = tmp + j;
			memmove(i + 3, i + 1, strlen(i + 1) + 1);
			*i++ = '%';
			*i++ = '2';
			*i = '0';
		}
	}
	return tmp;
}

int lt_submitentry(metadata meta)
{
	CURL *curl;
	int status;
	char lt_submit_url[4096] = "", *artist, *album, *title, *genre = NULL, *year = NULL, *length, tmp[256] = "";

	artist = malloc(wcslen(meta.artist) + 1);
	memset(artist, '\0', wcslen(meta.artist) + 1);
	wcstombs(artist, meta.artist, wcslen(meta.artist));
	artist = adjust_spaces(artist);
	album = malloc(wcslen(meta.album) + 1);
	memset(album, '\0', wcslen(meta.album) + 1);
	wcstombs(album, meta.album, wcslen(meta.album));
	album = adjust_spaces(album);
	title = malloc(wcslen(meta.title) + 1);
	memset(title, '\0', wcslen(meta.title) + 1);
	wcstombs(title, meta.title, wcslen(meta.title));
	title = adjust_spaces(title);
	if(meta.genre != NULL)
	{
		genre = malloc(wcslen(meta.genre) + 1);
		memset(genre, '\0', wcslen(meta.genre) + 1);
		wcstombs(genre, meta.genre, wcslen(meta.genre));
		genre = adjust_spaces(genre);
	}
	if(meta.year != NULL)
	{
		year = malloc(wcslen(meta.year) + 1);
		memset(year, '\0', wcslen(meta.year) + 1);
		wcstombs(year, meta.year, wcslen(meta.year));
		year = adjust_spaces(year);
	}
	length = malloc(21);
	memset(length, '\0', 21);
	snprintf(length, 20, "%d", meta.length);

	snprintf(lt_submit_url, sizeof(lt_submit_url), "%s/postmp3.php?USER=%s&PASS=%s&ARTIST=%s&ALBUM=%s&TITLE=%s&LENGTH=%d", LISTENTO_URL, lt_username, lt_password, artist, album, title, meta.length);
	if(meta.genre != NULL)
	{
		snprintf(tmp, sizeof(tmp), "&GENRE=%s", genre);
		strncat(lt_submit_url, tmp, sizeof(lt_submit_url) - strlen(tmp));
	}
	if(meta.year != NULL)
	{
		snprintf(tmp, sizeof(tmp), "&YEAR=%s", year);
		strncat(lt_submit_url, tmp, sizeof(lt_submit_url) - strlen(tmp));
	}

	curl = curl_easy_init();
	curl_easy_setopt(curl, CURLOPT_NOPROGRESS, 1);
	curl_easy_setopt(curl, CURLOPT_URL, lt_submit_url);
	curl_easy_setopt(curl, CURLOPT_USERAGENT, LISTENTO_CLI_ID);
	pdebug(fmt_vastr("ARTIST=%s ALBUM=%s TITLE=%s GENRE=%s YEAR=%s LENGTH=%s",
			artist,
			album,
			title,
			genre,
			year,
			length));
	memset(lt_curl_errbuf, 0, sizeof(lt_curl_errbuf));
	curl_easy_setopt(curl, CURLOPT_ERRORBUFFER, lt_curl_errbuf);
	status = curl_easy_perform(curl);
	curl_easy_cleanup(curl);

	free(artist);
	free(album);
	free(title);
	if(meta.genre != NULL) free(genre);
	if(meta.year != NULL) free(year);
	free(length);

	if (status) {
		pdebug(lt_curl_errbuf);
		lt_errors++;
		return -1;
	}
	return 0;
}

static void lt_cleaner(void)
{
	free(lt_username);
	/* free(sc_srv_res);
	dump_queue();
	q_free();*/
	pdebug("listen-to shutting down");
}

void lt_init(char *uname, char *pwd)
{
	lt_username = strdup(uname);
	lt_password = strdup(pwd);
	atexit(lt_cleaner);
	pdebug("listen-to starting up");
}

int lt_idle(void)
{
	return 0;
}
